import 'dart:io';
import 'dart:typed_data';

import 'package:convert_native_img_stream/convert_native_img_stream.dart';
import 'package:image/image.dart' as img;
import 'package:flutter/material.dart';
import 'package:google_mlkit_text_recognition/google_mlkit_text_recognition.dart';

import 'package:path_provider/path_provider.dart';
import 'package:reader_ocr/widgets/painter/interest_zone.dart';

import 'widgets/camera_viewer.dart';

class OcrViewer extends StatefulWidget {
  final Function(String? value) onValueReaded;
  const OcrViewer({super.key, required this.onValueReaded});

  @override
  State<OcrViewer> createState() => _OcrViewerState();
}

class _OcrViewerState extends State<OcrViewer> {
  String? value;

  @override
  Widget build(BuildContext context) {
    const double heightInterestFactor = 0.2;
    const double widthInterestFactor = 0.7;

    bool isProcessing = false;
    TextRecognizer textRecognizer =
        TextRecognizer(script: TextRecognitionScript.latin);

    void setValue(String text) {
      value = text;
      setState(() {});
    }

    Future<InputImage> cropImage(InputImage inputImage) async {
      var converterImage = ConvertNativeImgStream();

      Uint8List? uint8list = await converterImage.convertImgToBytes(
          inputImage.bytes!,
          inputImage.metadata!.size.width.toInt(),
          inputImage.metadata!.size.height.toInt());

      img.Image? convertedImage = img.decodeJpg(uint8list ?? Uint8List(0));

      if (convertedImage != null) {
        Directory directory = await getApplicationDocumentsDirectory();

        File file =
            await File("${directory.path}/arquivo.jpg").create(recursive: true);
        file.writeAsBytesSync(img.encodeJpg(img.copyCrop(
          convertedImage,
          x: (inputImage.metadata!.size.height -
                  (inputImage.metadata!.size.height * widthInterestFactor)) ~/
              2,
          y: (inputImage.metadata!.size.width -
                  (inputImage.metadata!.size.width * heightInterestFactor)) ~/
              2,
          height:
              (inputImage.metadata!.size.width * heightInterestFactor).toInt(),
          width:
              (inputImage.metadata!.size.height * widthInterestFactor).toInt(),
        )));

        return InputImage.fromFile(file);
      } else {
        return inputImage;
      }
    }

    String filterRecognizedText(String recognizedText) {
      RegExp alphaRegex = RegExp(r'[^0-9.]');

      String filteredText =
          recognizedText.replaceAll(",", ".").replaceAll(alphaRegex, "");

      return filteredText;
    }

    Future<void> analyseImage(InputImage inputImage) async {
      isProcessing = true;

      RecognizedText textRecognized =
          await textRecognizer.processImage(await cropImage(inputImage));

      setValue(filterRecognizedText(textRecognized.text));

      widget.onValueReaded(value);
      isProcessing = false;
    }

    void onImageStreamed(InputImage? cameraImage) {
      if (cameraImage != null && isProcessing == false) {
        analyseImage(cameraImage);
      }
    }

    return Scaffold(
      body: CameraViewer(
        onImageStreamed: onImageStreamed,
        widget: Center(
            child: CustomPaint(
          painter: InterestZone(
              heightInterestFactor: heightInterestFactor,
              widthInterestFactor: widthInterestFactor),
          child: Visibility(
              visible: value != null,
              child: Center(
                child: Text(value ?? "",
                    style: const TextStyle(color: Colors.lightBlue)),
              )),
        )),
      ),
    );
  }
}
